//
//  DBIDCardData.m
//  ble_nfc_sdk
//
//  Created by h_n on 2019/5/5.
//  Copyright © 2019年 Lochy. All rights reserved.
//

#import "DBIDCardData.h"
#import "NSData+Hex.h"

@implementation DBIDCardData

extern unsigned char *CC_MD5(const void *data, uint32_t len, unsigned char *md)
__OSX_AVAILABLE_STARTING(__MAC_10_4, __IPHONE_2_0);

static int ID_TYPE_CN = 1;       //身份证类型-居民身份证
static int ID_TYPE_GAT = 2;      //身份证类型-港澳台居民身份证
static int ID_TYPE_FOREIGN = 3;  //身份证类型-外国人永久居留身份证

+ (instancetype)formatter:(NSData *)idCardData image:(NSData *)imageParam{
    if (idCardData.length < 1295) { // 数据正确应该是1296
        return nil;
    }
    
    DBIDCardData *idCard = [[DBIDCardData alloc] init];
    
    Byte *idCardBytes = (Byte *)idCardData.bytes;
    if ( (idCardBytes[0] == (Byte)0xaa)
        && (idCardBytes[1] == (Byte)0xaa)
        && (idCardBytes[2] == (Byte)0xaa)
        && (idCardBytes[3] == (Byte)0x96)
        && (idCardBytes[4] == (Byte)0x69)) {
        
        //int totalLen = ((idCardBytes[5] & 0xff) << 8) | (idCardBytes[6] & 0xff);
        int wordMsgBytesLen = ((idCardBytes[10] & 0xff) << 8) | (idCardBytes[11] & 0xff);
        int photoMsgBytesLen = ((idCardBytes[12] & 0xff) << 8) | (idCardBytes[13] & 0xff);
        
        NSData *wordMsg;
        NSData *photo;
        NSData *fingerprinter;
        
        Byte *wordMsgBytes; // 文本信息
        Byte *photoMsgBytes; // 人像信息
        Byte *fingerprintBytes; // 指纹信息
        
        if (idCardData.length == 1295) {   //不带指纹
            //System.arraycopy(idCardBytes, 14, wordMsgBytes, 0, wordMsgBytesLen); // 文本信息
            //System.arraycopy(idCardBytes, 14 + wordMsgBytesLen, photoMsgBytes, 0, photoMsgBytesLen); // 图像
            wordMsg = [idCardData subdataWithRange:NSMakeRange(14, wordMsgBytesLen)];
            photo   = [idCardData subdataWithRange:NSMakeRange(14 + wordMsgBytesLen, photoMsgBytesLen)];
            wordMsgBytes = (Byte *)wordMsg.bytes;
            photoMsgBytes = (Byte *)photo.bytes;
        } else {   //带指纹
            int fingerprintBytesLen = ((idCardBytes[14] & 0xff) << 8) | (idCardBytes[15] & 0xff);   //指纹长度
            //fingerprintBytes = new byte[fingerprintBytesLen];
            //System.arraycopy(idCardBytes, 16, wordMsgBytes, 0, wordMsgBytesLen); // 文本信息
            //System.arraycopy(idCardBytes, 16 + wordMsgBytesLen, photoMsgBytes, 0, photoMsgBytesLen); //人像
            //System.arraycopy(idCardBytes, 16 + wordMsgBytesLen + photoMsgBytesLen, fingerprintBytes, 0, fingerprintBytesLen); // 指纹
            wordMsg         = [idCardData subdataWithRange:NSMakeRange(16, wordMsgBytesLen)];
            photo           = [idCardData subdataWithRange:NSMakeRange(16 + wordMsgBytesLen, photoMsgBytesLen)];
            fingerprinter   = [idCardData subdataWithRange:NSMakeRange(16 + wordMsgBytesLen + photoMsgBytesLen, fingerprintBytesLen)];
            
            wordMsgBytes     = (Byte *)wordMsg.bytes;
            photoMsgBytes    = (Byte *)photo.bytes;
            fingerprintBytes = (Byte *)fingerprinter.bytes;
        }
        
        //判断身份证的类型是否为港澳台身份证
        if (wordMsgBytes[248] == 'J') {
            idCard.type = ID_TYPE_GAT;
        } else if (wordMsgBytes[248] == 'I') {
            idCard.type = ID_TYPE_FOREIGN;
        } else {
            idCard.type = ID_TYPE_CN;
        }
        
        int index = 0;
        int nameLength = 30;
        NSData *nameData = [wordMsg subdataWithRange:NSMakeRange(index, 30)];
        idCard.fullName = [[NSString alloc] initWithData:nameData encoding:NSUTF16LittleEndianStringEncoding];
        index += nameLength;
        
        //性别
        if (wordMsgBytes[30] == 0x31) {
            idCard.gener = @"男";
        } else {
            idCard.gener = @"女";
        }
        index += 2;
        
        //名族
        int nationLength = 4;
        if (idCard.type == ID_TYPE_CN) {
            NSData *nationData = [wordMsg subdataWithRange:NSMakeRange(index, nationLength)];
            NSString *nationCode = [[NSString alloc] initWithData:nationData encoding:NSUTF16LittleEndianStringEncoding];
            idCard.nation = [DBIDCardData getNationString:[nationCode intValue]];
        }
        index += nationLength;
        
        //出生
        int birthdayLength = 16;
        NSData *birthdayData = [wordMsg subdataWithRange:NSMakeRange(index, birthdayLength)];
        idCard.birthday = [[NSString alloc] initWithData:birthdayData encoding:NSUTF16LittleEndianStringEncoding];
        index += birthdayLength;
        
        // 住址
        int addressLength = 70;
        NSData *addressData = [wordMsg subdataWithRange:NSMakeRange(index, addressLength)];
        idCard.address = [[NSString alloc] initWithData:addressData encoding:NSUTF16LittleEndianStringEncoding];
        index += addressLength;
        
        // 身份号码
        int idNumberLength = 36;
        NSData *idNumberData = [wordMsg subdataWithRange:NSMakeRange(index, idNumberLength)];
        idCard.idNumber = [[NSString alloc] initWithData:idNumberData encoding:NSUTF16LittleEndianStringEncoding];
        index += idNumberLength;
        
        // 签发机关
        int grantDeptLength = 30;
        NSData *grantDeptData = [wordMsg subdataWithRange:NSMakeRange(index, grantDeptLength)];
        idCard.grantDept = [[NSString alloc] initWithData:grantDeptData encoding:NSUTF16LittleEndianStringEncoding];
        index += grantDeptLength;
        
        // 有效起始日期
        int startDateLength = 16;
        NSData *startDateData = [wordMsg subdataWithRange:NSMakeRange(index, startDateLength)];
        idCard.startDate = [[NSString alloc] initWithData:startDateData encoding:NSUTF16LittleEndianStringEncoding];
        index += startDateLength;
        
        
        // 有效起始日期
        int endDateLength = 16;
        NSData *endDateData = [wordMsg subdataWithRange:NSMakeRange(index, endDateLength)];
        idCard.endDate = [[NSString alloc] initWithData:endDateData encoding:NSUTF16LittleEndianStringEncoding];
        index += endDateLength;
        
        
        // 港澳台身份证
        if (idCard.type == ID_TYPE_GAT) {
            // 护照号
            int passportLength = 16;
            NSData *passportData = [wordMsg subdataWithRange:NSMakeRange(index, passportLength)];
            idCard.passport = [[NSString alloc] initWithData:passportData encoding:NSUTF16LittleEndianStringEncoding];
            index += passportLength;
            
            // 签发次数
            int issueNumberLength = 16;
            NSData *issueNumberData = [wordMsg subdataWithRange:NSMakeRange(index, issueNumberLength)];
            idCard.issueNumber = [[NSString alloc] initWithData:issueNumberData encoding:NSUTF16LittleEndianStringEncoding];
            index += issueNumberLength;
        }
        
        //获取照片
        NSMutableDictionary *postData = [[NSMutableDictionary alloc] init];

        NSMutableString *dataString = [[NSMutableString alloc] init];
        [dataString appendString:@"{\"A0\":\"4b00"];
        [dataString appendString:[NSData hexStringFromData:imageParam]];
        [dataString appendString:@"1205040000"];
        [dataString appendString:[NSData hexStringFromData:idCardData]];
        [dataString appendString:@"\"}"];
        [postData setValue:[NSString stringWithString:dataString] forKey:@"data"];
        [postData setValue:@"10000000" forKey:@"appid"];
        
        NSString *timestampString = [[NSString alloc] initWithFormat:@"%llu", (UInt64)[[NSDate date] timeIntervalSince1970]];
        [postData setValue:timestampString forKey:@"timestamp"];
        
        NSMutableString *md5InString = [[NSMutableString alloc] init];
        [md5InString appendString:@"10000000"];
        [md5InString appendString:@"xnK6kGRwt9wIODab+TzsYAP3q/ZRujTjvZfcJAnC+1w="];
        [md5InString appendString:[NSString stringWithString:dataString]];
        [md5InString appendString:timestampString];
        NSString *signString = [self md5DigestWithString:[NSString stringWithString:md5InString]];
        [postData setValue:signString forKey:@"sign"];
        
//        NSLog(@"postData=%@", postData);
        NSDictionary *rspDic = [self postSyn:@"https://osspicdk.dkcloudid.cn/api/photo" jsonData:postData];
//        NSLog(@"rspDic=%@", rspDic);
        
        @try {
            idCard.imageData = [[NSData alloc] initWithBase64EncodedString:[rspDic objectForKey:@"data"]
                                                                    options:NSDataBase64DecodingIgnoreUnknownCharacters];
        } @catch (NSException *exception) {
            NSLog(@"获取照片失败");
        } @finally {
            
        }
        
        
        return idCard;
    } else {
        return nil;
    }
}

+ (NSString *)getNationString:(int)code {
    switch(code) {
        case 01:  return @"汉";
        case 02:  return @"蒙古";
        case 03:  return @"回";
        case 04:  return @"藏";
        case 05:  return @"维吾尔";
        case 06:  return @"苗";
        case 07:  return @"彝";
        case 8:   return @"壮";
        case 9:   return @"布依";
        case 10:  return @"朝鲜";
        case 11:  return @"满";
        case 12:  return @"侗";
        case 13:  return @"瑶";
        case 14:  return @"白";
        case 15:  return @"土家";
        case 16:  return @"哈尼";
        case 17:  return @"哈萨克";
        case 18:  return @"傣";
        case 19:  return @"黎";
        case 20:  return @"傈僳";
        case 21:  return @"佤";
        case 22:  return @"畲";
        case 23:  return @"高山";
        case 24:  return @"拉祜";
        case 25:  return @"水";
        case 26:  return @"东乡";
        case 27:  return @"纳西";
        case 28:  return @"景颇";
        case 29:  return @"柯尔克孜";
        case 30:  return @"土";
        case 31:  return @"达斡尔";
        case 32:  return @"仫佬";
        case 33:  return @"羌";
        case 34:  return @"布朗";
        case 35:  return @"撒拉";
        case 36:  return @"毛南";
        case 37:  return @"仡佬";
        case 38:  return @"锡伯";
        case 39:  return @"阿昌";
        case 40:  return @"普米";
        case 41:  return @"塔吉克";
        case 42:  return @"怒";
        case 43:  return @"乌孜别克";
        case 44:  return @"俄罗斯";
        case 45:  return @"鄂温克";
        case 46:  return @"德昂";
        case 47:  return @"保安";
        case 48:  return @"裕固";
        case 49:  return @"京";
        case 50:  return @"塔塔尔";
        case 51:  return @"独龙";
        case 52:  return @"鄂伦春";
        case 53:  return @"赫哲";
        case 54:  return @"门巴";
        case 55:  return @"珞巴";
        case 56:  return @"基诺";
        case 97:  return @"其他";
        case 98:  return @"外国血统中国籍人士";
        default : return @"";
    }
}

+ (NSString *)idCardDataToString:(DBIDCardData *)idCardData{
    if (idCardData.type == ID_TYPE_GAT) {
        NSMutableString *strTmp = [[NSMutableString alloc] init];
        [strTmp appendFormat:@"\r\n姓        名：%@", idCardData.fullName];
        [strTmp appendFormat:@"\r\n性        别：%@", idCardData.gener];
        [strTmp appendFormat:@"\r\n出生日期：%@", idCardData.birthday];
        [strTmp appendFormat:@"\r\n住        址：%@", idCardData.address];
        [strTmp appendFormat:@"\r\n身份 证号：%@", idCardData.idNumber];
        [strTmp appendFormat:@"\r\n签发 机关：%@", idCardData.grantDept];
        [strTmp appendFormat:@"\r\n有  效  期：%@-%@", idCardData.startDate, idCardData.endDate];
        [strTmp appendFormat:@"\r\n通行 证号：%@", idCardData.passport];
        [strTmp appendFormat:@"\r\n签发 次数：%@", idCardData.issueNumber];
        return strTmp;
    }
    else {
        NSMutableString *strTmp = [[NSMutableString alloc] init];
        [strTmp appendFormat:@"\r\n姓        名：%@", idCardData.fullName];
        [strTmp appendFormat:@"\r\n性        别：%@", idCardData.gener];
        [strTmp appendFormat:@"\r\n名        族：%@", idCardData.nation];
        [strTmp appendFormat:@"\r\n出生日期：%@", idCardData.birthday];
        [strTmp appendFormat:@"\r\n住        址：%@", idCardData.address];
        [strTmp appendFormat:@"\r\n身份 证号：%@", idCardData.idNumber];
        [strTmp appendFormat:@"\r\n签发 机关：%@", idCardData.grantDept];
        [strTmp appendFormat:@"\r\n有  效  期：%@-%@", idCardData.startDate, idCardData.endDate];
        return strTmp;
    }
}

//同步post
+(NSDictionary *)postSyn:(NSString *)urlStr jsonData:(NSMutableDictionary *)mulDic{
    NSString *reqData = @"";
    NSData *postDatas = nil;
    NSString *urlPath = urlStr;
     
    //NSJSONSerialization 组json字符串
    if ([NSJSONSerialization isValidJSONObject:mulDic]) {
        postDatas = [NSJSONSerialization dataWithJSONObject:mulDic options:NSJSONWritingPrettyPrinted error:nil];
        NSString *str = [[NSString alloc] initWithData:postDatas encoding:NSUTF8StringEncoding];
        reqData = [reqData stringByAppendingString:str];
        NSLog(@"reqData:%@",reqData);
         
        //NSData *postData = [reqData dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES];
        postDatas = [NSData dataWithBytes:[reqData UTF8String] length:[reqData length]];
    }
     
    NSString *len = [NSString stringWithFormat:@"%d",(int)[postDatas length]];
    NSURL *url = [NSURL URLWithString:urlPath];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    [request setHTTPMethod:@"POST"];
    [request setTimeoutInterval: 20]; //设置连接超时
    [request setValue:len forHTTPHeaderField:@"Content-Length"]; //设置数据长度
    [request setValue:@"application/json" forHTTPHeaderField:@"Content-Type"]; //设置发送数据的格式
    [request setValue:@"application/json" forHTTPHeaderField:@"Accept"]; //设置预期接收数据的格式
    [request setHTTPBody:postDatas];
     
    NSData *retData = [NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil];
    NSString *ret = [[NSString alloc] initWithData:retData encoding:NSUTF8StringEncoding];
    NSLog(@"%@",ret);
     
    //解析返回的数据JSON格式字符串
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:retData options:NSJSONReadingMutableLeaves error:nil];
    //NSLog(@"%@",dic);
    
    return dic;
}

+(NSString *)md5DigestWithString:(NSString*)input{
    const char* str = [input UTF8String];
    unsigned char result[16];
    CC_MD5(str, (uint32_t)strlen(str), result);
    NSMutableString *ret = [NSMutableString stringWithCapacity:16 * 2];
    for(int i = 0; i<16; i++) {
        [ret appendFormat:@"%02x",(unsigned int)(result[i])];
    }
    return ret;
}

@end
