//
//  DKCloudID.m
//  ble_nfc_sdk
//
//  Created by lochy on 2019/5/7.
//  Copyright © 2019 Lochy. All rights reserved.
//

#import "DKCloudID.h"
#import "DBStreamDataTool.h"
#import "DKServerException.h"

static NSString     *socketHost =  @"www.dkcloudid.cn";  //@"192.168.3.198"; //
static NSInteger    socketPort = 20006;
static int          connectToServerTimeOutMs = 6000;


@interface DKCloudID () <NSStreamDelegate>

@end

@implementation DKCloudID
@synthesize inputStream;
@synthesize outputStream;
@synthesize isWriteDataAvailable;

//接收到数据回调
static onReceiveDataListener onReceiveDataListenerBlock = nil;
static connectServerSuccessListener connectServerSuccessBlock = nil;

int dataLength = 0;
NSData *returnData;

- (id)init{
    self = [super init];
    if (self){
        inputStream = nil;
        outputStream = nil;
        isWriteDataAvailable = NO;
        
        dispatch_semaphore_t sema = dispatch_semaphore_create(0);
        [self connectToServer:socketHost port:socketPort success:^{
            dispatch_semaphore_signal(sema);   //释放信号量
        }];
        
        if ( dispatch_semaphore_wait(sema, dispatch_time(DISPATCH_TIME_NOW, connectToServerTimeOutMs*NSEC_PER_MSEC)) != 0 ) {  //等待信号量
            self = nil;
        }
    }
    return self;
}

//建立与服务器的连接
- (void)connectToServer:(NSString *)host port:(NSInteger)port {
    
    // 创建CF下的读入流
    CFReadStreamRef readStream;
    // 创建CF下的写出流
    CFWriteStreamRef writeStream;
    
    // 创建流
    CFStreamCreatePairWithSocketToHost(NULL, (__bridge CFStringRef)(host), (int)port, &readStream, &writeStream);
    
    // 将CFXXX流和NSXXX流建立对应关系
    inputStream = (__bridge NSInputStream *)(readStream);
    outputStream = (__bridge NSOutputStream *)(writeStream);
    
    // 设置通信过程中的代理
    inputStream.delegate = self;
    outputStream.delegate = self;
    
    // 将流对象添加到主运行循环(如果不加到主循环,Socket流是不会工作的)
    [inputStream scheduleInRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    [outputStream scheduleInRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    
    // 打开流
    [inputStream open];
    [outputStream open];
}

//建立与服务器的连接
- (void)connectToServer:(NSString *)host port:(NSInteger)port success:(connectServerSuccessListener)block {
    connectServerSuccessBlock = block;
    [self connectToServer:host port:port];
}

//断开链接
- (void)close{
    //关闭流
    if (inputStream) {
        [inputStream close];
        [inputStream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    }
    
    if (outputStream) {
        [outputStream close];
        [outputStream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
    }
    
    isWriteDataAvailable = NO;
    inputStream = nil;
    outputStream = nil;
}

- (void)stream:(NSStream *)aStream handleEvent:(NSStreamEvent)eventCode {
    //NSLog(@"%@",eventCode);
    
    switch (eventCode) {
        case NSStreamEventOpenCompleted:
//            NSLog(@"连接完成");
            break;
            
        case NSStreamEventHasBytesAvailable:
            [self readStreamData];
            break;
            
        case NSStreamEventHasSpaceAvailable:
//            NSLog(@"可以写入数据");
            isWriteDataAvailable = YES;
            if (connectServerSuccessBlock) {
                connectServerSuccessBlock();
            }
            break;
            
        case NSStreamEventErrorOccurred:
//            NSLog(@"发生错误");
            break;
            
        case NSStreamEventEndEncountered:
//            NSLog(@"流结束");
            isWriteDataAvailable = NO;
            
            // 做善后工作
            // 关闭流的同时，将流从主运行循环中删除
            [aStream close];
            [aStream removeFromRunLoop:[NSRunLoop mainRunLoop] forMode:NSDefaultRunLoopMode];
            break;
            
        default:
            break;
    }
}

/**
 * 发送数据到服务器，异步回调方式
 * @param sendData - 要发送到数据
 * @param block - 服务器返回到结果会回调到block
 */
- (void)sendPacket:(NSData *)sendData callBack:(onReceiveDataListener)block {
    if (outputStream) {
        onReceiveDataListenerBlock = block;
        NSData *header = [DBStreamDataTool header:(int)sendData.length];
        NSData *body = [DBStreamDataTool mergeData:header source:sendData begin:0 end:0];
        
        dataLength = 0;
        [outputStream write:body.bytes maxLength:body.length];
    }
}

/**
 * 发送数据到服务器，同步阻塞方式
 * @param sendData - 要发送到数据
 * @param timeMs - 超时等待时间，单位ms
 * @throws DKServerException
 *                  数据接收超时时会抛出异常
 */
- (NSData *)sendPacket:(NSData *)sendData timeout:(int)timeMs{
    if (!outputStream) {
        @throw [DKServerException
                exceptionWithName: @"DKServerException"
                reason: @"outputStream为空"
                userInfo: nil];
    }
    
    if (!isWriteDataAvailable) {
        @throw [DKServerException
                exceptionWithName: @"DKServerException"
                reason: @"数据发送未就绪"
                userInfo: nil];
    }
    
    //创建信号量
    __block NSData *returnDataTemp = nil;
    dispatch_semaphore_t sema = dispatch_semaphore_create(0);
    [self sendPacket:sendData callBack:^(NSData * _Nonnull data) {
        returnDataTemp = data;
        dispatch_semaphore_signal(sema);   //释放信号量
    }];

    if ( dispatch_semaphore_wait(sema, dispatch_time(DISPATCH_TIME_NOW, timeMs*NSEC_PER_MSEC)) != 0 ) {  //等待信号量
        //等待信号量超时
        @throw [DKServerException
                exceptionWithName: @"DKServerException"
                reason: @"数据接收超时"
                userInfo: nil];
    }

    return returnDataTemp;
}

//接收数据
- (void)readStreamData {
    if (inputStream == nil) {
        return;
    }
    
//    NSLog(@"有刻度字节");

    uint8_t buf[10240];
    //返回实际装的字节数
    NSInteger len = [inputStream read:buf maxLength:sizeof(buf)];
    
    if (len <= 0) {
        return;
    }
    
    //把字节数组转化成字符串
    NSData *data =[NSData dataWithBytes:buf length:len];
    if (data.length == 0) {
        return;
    }

    if (dataLength == 0) {
        dataLength = [DBStreamDataTool dataLength:data];
        returnData = data;
        if (returnData.length == dataLength) {
            dataLength = 0;
//            NSLog(@"本次数据接收完成");
            //NSLog(@"服务器返回内容:%@", returnData);
            
            //回调接收到的数据
            if (onReceiveDataListenerBlock) {
                //去掉头
                NSRange range = NSMakeRange(2, returnData.length - 2);
                NSData *dataTemp = [returnData subdataWithRange:range];
                onReceiveDataListenerBlock(dataTemp);
            }
        }
    } else {
        NSMutableData *temp = [returnData mutableCopy];
        [temp appendData:data];
        returnData = temp;

        if (returnData.length == dataLength) {
            dataLength = 0;
//            NSLog(@"本次数据接收完成");
            //NSLog(@"服务器返回内容:%@", returnData);
            
            //回调接收到的数据
            if (onReceiveDataListenerBlock) {
                //去掉头
                NSRange range = NSMakeRange(2, returnData.length - 2);
                NSData *dataTemp = [returnData subdataWithRange:range];
                onReceiveDataListenerBlock(dataTemp);
            }
        }
    }
}

@end
