//
//  SamVIdCard.m
//  ble_nfc_sdk
//
//  Created by lochy on 2019/4/24.
//  Copyright © 2019 Lochy. All rights reserved.
//

#import "SamVIdCard.h"
#import "DKCardNoResponseException.h"

@implementation SamVIdCard

/**
 * 获取解析初始化数据，同步阻塞方式，注意：不能在主线程里运行
 * @return         返回解析初始化数据
 * @throws CardNoResponseException
 *                  操作无响应时会抛出异常
 */
-(NSData *)getSamVInitData{
    //创建信号量
    __block BOOL status = NO;
    __block NSData *returnDataTemp = nil;
    dispatch_semaphore_t sema = dispatch_semaphore_create(0);
    [self.deviceManager requestSamVInitData:^(BOOL isSuc, NSData *samVInitData) {
        status = isSuc;
        returnDataTemp = samVInitData;
        dispatch_semaphore_signal(sema);   //释放信号量
    }];
    if ( dispatch_semaphore_wait(sema, dispatch_time(DISPATCH_TIME_NOW,5*CAR_NO_RESPONSE_TIME_MS*NSEC_PER_MSEC)) != 0 ) {  //等待信号量
        //等待信号量超时
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"卡片无响应"
                userInfo: nil];
    }
    else if (!status) {
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"获取解析初始化数据失败"
                userInfo: nil];
    }
    else {
        return returnDataTemp;
    }
}

/**
 * 获取解析结束后用于解密明文的aeskey，同步阻塞方式，注意：不能在主线程里运行
 * @return         返回的aes key，16字节
 * @throws CardNoResponseException
 *                  操作无响应时会抛出异常
 */
-(NSData *)getSamVAesKeyData{
    //创建信号量
    __block BOOL status = NO;
    __block NSData *returnDataTemp = nil;
    dispatch_semaphore_t sema = dispatch_semaphore_create(0);
    [self.deviceManager requestSamVGetAESKey:^(BOOL isSuc, NSData *aesKey) {
        status = isSuc;
        returnDataTemp = aesKey;
        dispatch_semaphore_signal(sema);   //释放信号量
    }];
    if ( dispatch_semaphore_wait(sema, dispatch_time(DISPATCH_TIME_NOW,5*CAR_NO_RESPONSE_TIME_MS*NSEC_PER_MSEC)) != 0 ) {  //等待信号量
        //等待信号量超时
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"设备无响应"
                userInfo: nil];
    }
    else if (!status) {
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"获取解析AES KEY失败"
                userInfo: nil];
    }
    else {
        return returnDataTemp;
    }
}

/**
 * 解析过程交互，注意：不能在蓝牙初始化的线程里运行
 * @param data     发送的数据
 * @return         返回的数据
 * @throws CardNoResponseException
 *                  操作无响应时会抛出异常
 */
-(NSData *)samVIdCardtransceive:(NSData *)data{
    //创建信号量
    __block BOOL status = NO;
    __block NSData *returnDataTemp = nil;
    dispatch_semaphore_t sema = dispatch_semaphore_create(0);
    [self.deviceManager requestSamVDataExchange:data callback:^(BOOL isSuc, NSData *apduData) {
        status = isSuc;
        returnDataTemp = apduData;
        dispatch_semaphore_signal(sema);   //释放信号量
    }];
    if ( dispatch_semaphore_wait(sema, dispatch_time(DISPATCH_TIME_NOW,5*CAR_NO_RESPONSE_TIME_MS*NSEC_PER_MSEC)) != 0 ) {  //等待信号量
        //等待信号量超时
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"设备无响应"
                userInfo: nil];
    }
    else if (!status) {
        @throw [DKCardNoResponseException
                exceptionWithName: @"DKCardNoResponseException"
                reason: @"apdu指令发送失败"
                userInfo: nil];
    }
    else {
        return returnDataTemp;
    }
}

@end
